/*
 AlarmSM 433MHz Remote Control Server
 Version: 0.1
 Author: Alex from insideGadgets (http://www.insidegadgets.com)
 Created: 22/12/2014
 Last Modified: 26/12/2014

 Listen for incoming 433MHz rolling codes from the remote and pass this on to the AlarmSM server.
 You can also sync up remotes by pressing the R1 / R2 buttons on the server and pressing a button on the remote.
 
 */

// ATtiny45/85 Pin map
//
//                       +-\/-+
//           Reset PB5  1|o   |8  VCC
//  Update R1 Code PB3  2|    |7  PB2 System state
//  Update R2 Code PB4  3|    |6  PB1 LED
//                 GND  4|    |5  PB0 433MHz In
//                       +----+

#define F_CPU 1000000 // 1 MHz

#define RF_IN PB0
#define LED PB1
#define SYSTEM_STATE PB2
#define UPDATE_RC1 PB3
#define UPDATE_RC2 PB4

#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/eeprom.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/sleep.h>
#include <avr/pgmspace.h>
#include <string.h>
#include "setup.c"
#include "hmac-sha1/hmac-sha1.h"

// 256bit random number for rolling code (change these for each remote)
const uint8_t random_number_r1_base[] PROGMEM = {0x27, 0x9B, 0x6D, 0xD8, 0xBA, 0x17, 0x62, 0xD7,
																0xBB, 0x14, 0xF7, 0xDF, 0xDA, 0x41, 0xEE, 0x66,
																0x8A, 0x7A, 0x8D, 0X6F, 0xFA, 0x4F, 0xDA, 0xFC,
																0x41, 0x01, 0x12, 0x3B, 0x9F, 0xF7, 0xF2, 0xAF};
const uint8_t random_number_r2_base[] PROGMEM = {0x27, 0x9B, 0x6D, 0xD8, 0xBA, 0x17, 0x62, 0xD7,
																0xBB, 0x14, 0xF7, 0xDF, 0xDA, 0x41, 0xEE, 0x66,
																0x8A, 0x7A, 0x8D, 0X6F, 0xFA, 0x4F, 0xDA, 0xFC,
																0x41, 0x01, 0x12, 0x3B, 0x9F, 0xF7, 0xF2, 0xAF};

uint8_t random_number_r1[32] = {0};									
uint8_t random_number_r2[32] = {0};
uint32_t random_number_r1_addition = 0;
uint32_t random_number_r2_addition = 0;

uint8_t random_number_check[32];
uint8_t updateRC = 0;

// Increment the rolling code in a temporary variable for testing higher rolling codes
void higher_rolling_code_check(uint32_t counter, uint8_t remoteNumber) {
	if (remoteNumber == 1) {
		for (uint8_t x = 0; x < 32; x++) {
			random_number_check[x] = random_number_r1[x];
		}
	}
	else {
		for (uint8_t x = 0; x < 32; x++) {
			random_number_check[x] = random_number_r2[x];
		}
	}
	
	// Increment our 256bit random number
	for (uint32_t c = 0; c < counter; c++) {
		random_number_check[31]++;
		if (random_number_check[31] == 0) {
			for (int8_t a = 31; a >= 0; a--) {
				if (random_number_check[a] == 0) {
					random_number_check[(a-1)]++;
				}
				else {
					break;
				}
			}
		}
	}
	
	// Generate the random block (160 bit) using SHA1 from our random number
	sha1(random_number_sha, random_number_check, 256);
}

int main(void) {
	setup();
	
	while(1) {
		// Receive data
		if (rf433_receive_data()) {
			PORTB |= (1<<LED); // Valid packet received
			_delay_ms(10);
			PORTB &= ~(1<<LED);
			
			// Check if buttons held for updating RC code on server
			if (PINB & (1<<UPDATE_RC1)) {
				updateRC = 1;
				while (PINB & (1<<UPDATE_RC1)) { // Wait for button release
					_delay_ms(250);
				}
			}
			else if (PINB & (1<<UPDATE_RC2)) {
				updateRC = 2;
				while (PINB & (1<<UPDATE_RC2)) { // Wait for button release
					_delay_ms(250);
				}
			}
			
			// Start updating RC code on server
			if (updateRC >= 1) {
				PORTB |= (1<<LED);
				uint32_t codeCounter = 1;
				
				// Init rolling code to 0
				if (updateRC == 1) {
					rc_code_init(random_number_r1, random_number_r1_base, 0);
				}
				else {
					rc_code_init(random_number_r2, random_number_r2_base, 0);
				}
				higher_rolling_code_check(codeCounter, updateRC);
				
				// Loop until we find the random number in use, press the button again to exit
				while (memcmp(data_in, random_number_sha, 20) != 0) {
					codeCounter++;
					higher_rolling_code_check(codeCounter, updateRC);
					
					// Break if button pressed
					if (PINB & (1<<UPDATE_RC1) || PINB & (1<<UPDATE_RC2)) {
						updateRC = 0;
						break;
					}
				}
				
				// Update RC codes
				if (updateRC == 1) {
					// Copy random number that worked to current random number
					for (uint8_t x = 0; x < 32; x++) {
						random_number_r1[x] = random_number_check[x];
					}
					
					// Increment the rolling code addition to tell remotes which random number to use
					random_number_r1_addition = codeCounter;
					
					// Update on EEPROM
					eeprom_write_byte((uint8_t*) RAND_R1_ADDITION_LOCATION, (uint8_t) ((uint32_t) random_number_r1_addition >> 24));
					eeprom_write_byte((uint8_t*) RAND_R1_ADDITION_LOCATION+1, (uint8_t) ((uint32_t) random_number_r1_addition >> 16));
					eeprom_write_byte((uint8_t*) RAND_R1_ADDITION_LOCATION+2, (uint8_t) ((uint32_t) random_number_r1_addition >> 8));
					eeprom_write_byte((uint8_t*) RAND_R1_ADDITION_LOCATION+3, (uint8_t) random_number_r1_addition & 0xFF);
					
					// Update R1Set
					if (R1Set == BLANK_EEPROM_BYTE) {
						eeprom_write_byte((uint8_t*) R1_SET_LOCATION, 0);
						R1Set = 0;
					}
				}
				else {
					// Copy random number that worked to current random number
					for (uint8_t x = 0; x < 32; x++) {
						random_number_r2[x] = random_number_check[x];
					}
					
					// Increment the rolling code addition to tell remotes which random number to use
					random_number_r2_addition = codeCounter;
					
					// Update on EEPROM
					eeprom_write_byte((uint8_t*) RAND_R2_ADDITION_LOCATION, (uint8_t) ((uint32_t) random_number_r2_addition >> 24));
					eeprom_write_byte((uint8_t*) RAND_R2_ADDITION_LOCATION+1, (uint8_t) ((uint32_t) random_number_r2_addition >> 16));
					eeprom_write_byte((uint8_t*) RAND_R2_ADDITION_LOCATION+2, (uint8_t) ((uint32_t) random_number_r2_addition >> 8));
					eeprom_write_byte((uint8_t*) RAND_R2_ADDITION_LOCATION+3, (uint8_t) random_number_r2_addition & 0xFF);
					
					// Update R2Set location
					if (R2Set == BLANK_EEPROM_BYTE) {
						eeprom_write_byte((uint8_t*) R2_SET_LOCATION, 0);
						R2Set = 0;
					}
				}
				
				PORTB &= ~(1<<LED);
				updateRC = 0;
				_delay_ms(250);
			}
			else {
				// Verify SHA1 (160 bit) to the client's SHA1
				uint8_t codeCounter = 1;
				uint8_t remoteControlNumber = 1;
				while (codeCounter < 30) {
					higher_rolling_code_check(codeCounter, remoteControlNumber);
					if (memcmp(data_in, random_number_sha, 20) == 0) {
						// Copy random number that worked to current random number
						if (remoteControlNumber == 1) {
							for (uint8_t x = 0; x < 32; x++) {
								random_number_r1[x] = random_number_check[x];
							}
							
							// Increment the rolling code addition to tell remotes which random number to use
							random_number_r1_addition = random_number_r1_addition + codeCounter;
							
							// Update on EEPROM
							eeprom_write_byte((uint8_t*) RAND_R1_ADDITION_LOCATION, (uint8_t) ((uint32_t) random_number_r1_addition >> 24));
							eeprom_write_byte((uint8_t*) RAND_R1_ADDITION_LOCATION+1, (uint8_t) ((uint32_t) random_number_r1_addition >> 16));
							eeprom_write_byte((uint8_t*) RAND_R1_ADDITION_LOCATION+2, (uint8_t) ((uint32_t) random_number_r1_addition >> 8));
							eeprom_write_byte((uint8_t*) RAND_R1_ADDITION_LOCATION+3, (uint8_t) random_number_r1_addition & 0xFF);
							
							// Update R1Set
							if (R1Set == BLANK_EEPROM_BYTE) {
								eeprom_write_byte((uint8_t*) R1_SET_LOCATION, 0);
								R1Set = 0;
							}
						}
						else {
							for (uint8_t x = 0; x < 32; x++) {
								random_number_r2[x] = random_number_check[x];
							}
							
							// Increment the rolling code addition to tell remotes which random number to use
							random_number_r2_addition = random_number_r2_addition + codeCounter;
							
							// Update on EEPROM
							eeprom_write_byte((uint8_t*) RAND_R2_ADDITION_LOCATION, (uint8_t) ((uint32_t) random_number_r2_addition >> 24));
							eeprom_write_byte((uint8_t*) RAND_R2_ADDITION_LOCATION+1, (uint8_t) ((uint32_t) random_number_r2_addition >> 16));
							eeprom_write_byte((uint8_t*) RAND_R2_ADDITION_LOCATION+2, (uint8_t) ((uint32_t) random_number_r2_addition >> 8));
							eeprom_write_byte((uint8_t*) RAND_R2_ADDITION_LOCATION+3, (uint8_t) random_number_r2_addition & 0xFF);
							
							// Update R2Set location
							if (R2Set == BLANK_EEPROM_BYTE) {
								eeprom_write_byte((uint8_t*) R2_SET_LOCATION, 0);
								R2Set = 0;
							}
						}
						
						// Process action
						if (remote_control_state == SYSTEM_ON) {
							PORTB |= (1<<LED);
							PORTB |= (1<<SYSTEM_STATE);
							_delay_ms(250);
							_delay_ms(100);
							PORTB &= ~(1<<LED);
							PORTB &= ~(1<<SYSTEM_STATE);
						}
						else if (remote_control_state == SYSTEM_OFF) {
							PORTB |= (1<<LED);
							PORTB |= (1<<SYSTEM_STATE);
							_delay_ms(250);
							_delay_ms(400);
							PORTB &= ~(1<<LED);
							PORTB &= ~(1<<SYSTEM_STATE);
						}
						else if (remote_control_state == SYSTEM_TEST) {
							PORTB |= (1<<LED);
							PORTB |= (1<<SYSTEM_STATE);
							_delay_ms(250);
							_delay_ms(800);
							PORTB &= ~(1<<LED);
							PORTB &= ~(1<<SYSTEM_STATE);
						}
						
						break;
					}
					codeCounter++;
					
					// Switch to other remote to test
					if (codeCounter >= 29 && remoteControlNumber == 1) {
						remoteControlNumber = 2;
						codeCounter = 1;
					}
				}
			}
		}
	}
}
